import requests
from requests import Request, Session
import logging
import simplejson as json

log = logging.getLogger('agent.client')


class DataApiClient():
    def __init__(self, address, token, verify_cert, agent_id, version, timeout=30):
        self.address = address
        self.token = token
        self.verify_cert = verify_cert
        self.agent_id = agent_id
        self.version = version
        self.timeout = timeout

        if timeout is None:
            self.timeout = 30

        log.info("Client started to: {0}, timeout: {1} agent: {2}".format(address, self.timeout, agent_id))

    def send(self, resource, action, data=None):
        try:
            url = self.address + resource

            headers = {'authorization': 'Bearer ' + self.token,
                       'BP-Namespace': self.agent_id,
                       'BP-Version': self.version}

            log.debug("Sending to {0}, data {1}, header: {2}".format(url, data, headers))

            session = Session()
            if data:
                if isinstance(data, dict):
                    data = json.dumps(data)
                headers['content-type'] = 'application/json';
                request = Request(action, url, data=data, headers=headers).prepare()
            else:
                request = Request(action, url, headers=headers).prepare()
            r = session.send(request, verify=self.verify_cert, timeout=self.timeout)

            if r.status_code >= 400 and r.status_code < 500:
                log.warn("Ignoring invalid request to {0}: HTTP response error code: {1}. Message: {2}".
                         format(url, r.status_code, r.text))

            if r.status_code >= 500:
                raise APIServerException("HTTP response error code: {0}. Message: {1}".format(r.status_code, r.text))

            if r.status_code == 200:
                return r.text

        except Exception, e:
            if isinstance(e, APIServerException):
                raise e

            raise ClientException(e)

    def sendErrors(self, errors):
        try:
            data = dict(count=len(errors), errors=errors[:10])
            url = self.address + "/agents/" + self.agent_id + "/error"
            log.debug("Sending {0} errors back home".format(len(errors)))
            headers = {'content-type': 'application/json',
                       'authorization': 'Bearer ' + self.token,
                       'BP-Version': self.version}

            requests.post(url, data=json.dumps(data), headers=headers, verify=self.verify_cert, timeout=self.timeout)
        except Exception, e:
            log.warn("Can't send errors to server: %s" % e)
            #raise ErrorsTransmissionException(e)


class APIServerException(Exception):
    def __init__(self, message, ):

    # Call the base class constructor with the parameters it needs
        Exception.__init__(self, message)

class ClientException(Exception):
    def __init__(self, message, ):

    # Call the base class constructor with the parameters it needs
        Exception.__init__(self, message)


class InvalidActionExcetion(ClientException):
    def __init__(self, message, ):

    # Call the base class constructor with the parameters it needs
        Exception.__init__(self, message)


class ErrorsTransmissionException(Exception):
    def __init__(self, message, ):

    # Call the base class constructor with the parameters it needs
        Exception.__init__(self, message)

