import logging
import simplejson as json

from bigpanda.agent.core.client import ClientException
from bigpanda.agent.core.commands import AgentCommands, CommandNotFoundException

log = logging.getLogger('agent.governor')
MAX_RETRIES = 5

class Governor(object):
    def __init__(self, scheduler, agent_config, client):
        self.agent_id = agent_config['agent_id']

        self.client = client

        scheduler.add_interval_job(self.check_commands, seconds=300, name="commands")

        self.agent_commands = AgentCommands(agent_config, scheduler=scheduler)
        self.errorCount = 0

    def check_commands(self):
        endpoint = "/agents/{0}/commands".format(self.agent_id)
        try:
            log.info("Checking for commands to run")
            response = self.client.send(endpoint, "GET")
            if response is None:
                return
            self.errorCount = 0

            response = json.loads(response)
            command = response.get('command', None)

            if not command:
                log.info("No commands to run.")
                return

            command_name = command.get('name', None)
            command_args = command.get('args', None)

            if not command_name:
                log.error('Got invalid command object.')
                return
            # Run command and POST results back
            response = dict(command=command_name)
            try:
                self.agent_commands.run(command_name, command_args)
                response.update(status="ok", msg="Command finished")
            except Exception as e:
                log.error("%s: %s" % (command_name, e))
                response.update(status="failed", msg=str(e))

            self.client.send(endpoint, "POST", response)

        except ClientException, e:
            log.error(e)
            self.errorCount += 1

        if self.errorCount > MAX_RETRIES:
            log.info("Commands request failed more than {0} times. Restarting".format(MAX_RETRIES))
            self.agent_commands.restart_service
