from bigpanda.agent.core.spi.exceptions import PluginExecutionException

import logging
import simplejson as json
import sys

log = logging.getLogger("agent")


class Plugin(object):
    def __init__(self, name):
        self.errors = []
        self.name = name
        self._init()

    def execute(self, config, state=None):
        self.errors = []
        try:
            if not state:
                state = self._init_state()

            log.debug("Executing plugin logic {0}".format(self.name))
            result, state, done = self._execute(config, state)
            log.debug("Done executing plugin logic {0}".format(self.name))

            if result:
                result = json.dumps(result)

            return result, state, done, self.errors
        except Exception, e:
            et, ei, tb = sys.exc_info()
            raise PluginExecutionException, PluginExecutionException(e), tb

    def _execute(self, config, state):
        """
        [Template function] This is where the plugin logic goes.
        Should be overridden by the concrete implementation.
        """
        pass

    def _init(self):
        """
        [Template function] Initializes the plugin
        """
        pass

    def _init_state(self):
        """
        Initialized the plugin's state. Can be overridden by the concrete implementation
        in case a more specific initialization is required.
        """
        return {}

    def _push_error(self, message, **kwargs):
        error = dict(message=message, data=kwargs)
        self.errors.append(error)

    def get_name(self):
        return self.name