import re
from datetime import datetime
import time
from bigpanda.agent.core.spi.plugins import Plugin
import requests


class GraphitePlugin(Plugin):
    def init_state(self):
        return {"last_timestamp": None}

    def _execute(self, config, state):
        metrics = config['metrics']
        graphite_endpoint = config['graphite_endpoint']
        from_timestamp = state["last_timestamp"]
        current_round_minute = time.mktime(datetime.utcnow().timetuple()) - time.mktime(
            datetime.utcnow().timetuple()) % 60
        if from_timestamp != current_round_minute:
            all_metrics_data = []
            for metric in metrics:
                url = self._build_url(graphite_endpoint, from_timestamp, current_round_minute, metric)
                all_metrics_data.extend(self._fetch_metric_data(url))

                state["last_timestamp"] = current_round_minute

            return {'datapoints': all_metrics_data}, state
        else: return None, state

    def _build_url(self, graphite_endpoint, from_timestamp, until_timestamp, metric):
        base_url = "/render?format=json"
        if from_timestamp:
            base_url += "&from={0}&until={1}".format(self._to_graphite_time_format(from_timestamp),
                                                     self._to_graphite_time_format(until_timestamp))
        else:
            base_url += "&until={0}".format(self._to_graphite_time_format(until_timestamp))

        return graphite_endpoint + base_url + "&target=" + metric


    def _to_graphite_time_format(self, timestamp):
        return datetime.fromtimestamp(timestamp).strftime('%H:%M_%Y%m%d')

    def _fetch_metric_data(self, url):
        metrics_data = []

        r = requests.get(url)
        if r.status_code == 200:
            data = r.json()
            if data and len(data) > 0:
                full_metric_name = data[0]['target']
                m = re.match("^monitoring\.nagios\.([^\.]*)\.([^\.]+)", full_metric_name)
                if m:
                    host = m.group(1)
                    short_metric_name = m.group(2)

                    datapoints = data[0]['datapoints']
                    for datapoint in datapoints:
                        metrics_data.append(
                            {"name": "_".join(["server", short_metric_name]), 'timestamp': datapoint[1] * 1000,
                             'value': datapoint[0], 'host': host})
            return metrics_data
        else:
            print r.status_code

    def _load_config(self, config):
        graphite_api_endpoint = config["graphite_endpoint"]
        metrics_url = graphite_api_endpoint + "/metrics/find?format=completer&query=*"

        all_metrics = []
        should_continue = True

        while should_continue:
            metrics = self._fetch_available_metrics(metrics_url)
            should_continue = len(metrics) > 0
            if should_continue:
                all_metrics.extend(metrics)
                metrics_url += ".*"

        config['metrics'] = [metric[0] for metric in all_metrics if metric[1]]

        return config

    def _fetch_available_metrics(self, url):
        metrics = []
        r = requests.get(url)
        if r.status_code == 200:
            response_metrics = r.json()['metrics']
            for metric_in_response in response_metrics:
                metrics.append((metric_in_response['path'], metric_in_response['is_leaf'] == '1'))

        return metrics

