class ObjectCache(object):

    def __init__(self):
        self.service_groups_by_host_and_service = {}
        self.hostgroups_by_host = {}

    def get_hostgroups(self, host):
        return self.hostgroups_by_host.get(host)

    def get_servicegroups(self, host, service):
        host_services = self.service_groups_by_host_and_service.get(host)
        if host_services:
            return host_services.get(service)
        return None

import re
import os
import stat

from bigpanda.agent.utils.file_utils import changed
import logging

START_BLOCK_REGEX = re.compile("define (\w+) \{$")
END_BLOCK_REGEX = re.compile("^\s*\}$")
MEMBERS_REGEX = re.compile("\s*members\s(.+)")
HOSTGROUP_NAME_REGEX = re.compile("\s*hostgroup_name\s*(\S+)$")
SERVICEGROUP_NAME_REGEX = re.compile("\s*servicegroup_name\s*(\S+)$")


class ObjectCacheParser(object):

    def __init__(self):
        self.in_block = False
        self.block_type = None
        self.log = logging.getLogger('agent.plugins.nagios.state.cache.parser')

    def parse(self, object_cache_filename):

        cache = ObjectCache()

        f = open(object_cache_filename, 'r')
        for line in f:
            if not self.in_block:
                matches = START_BLOCK_REGEX.match(line)
                if matches:
                    self.block_type = matches.group(1)
                    self.in_block = True
                    self.context = {}
                    continue
            else:
                matches = END_BLOCK_REGEX.match(line)
                if matches:
                    self.block_type = None
                    self.in_block = False
                    self.context = None
                    continue
                else:
                    if self.block_type == 'hostgroup':
                        # We assume that the host group name appears before the host group's members
                        if not self.context.get('hostgroup_name'):
                            hostgroup_name = self._parse_hostgroup_name(line)
                            self.context['hostgroup_name'] = hostgroup_name
                        if not self.context.get('members'):
                            members = self._parse_members(line)
                            self.context['members'] = members

                        members = self.context.get('members', [])
                        for member in members:
                            if not cache.hostgroups_by_host.get(member):
                                cache.hostgroups_by_host[member] = []

                            hostgroups = cache.hostgroups_by_host[member]
                            hostgroups.append(hostgroup_name)

                    elif self.block_type == 'servicegroup':
                        # We assume that the service group name appears before the service group's members
                        if not self.context.get('servicegroup_name'):
                            servicegroup_name = self._parse_servicegroup_name(line)
                            self.context['servicegroup_name'] = servicegroup_name
                        if not self.context.get('members'):
                            members = self._parse_members(line)
                            host_service_tuples = zip(members[0::2], members[1::2])

                            self.context['members'] = host_service_tuples

                        members = self.context.get('members', [])
                        for member in members:
                            host = member[0]
                            service = member[1]
                            if not cache.service_groups_by_host_and_service.get(host):
                                cache.service_groups_by_host_and_service[host] = {}

                            if not cache.service_groups_by_host_and_service.get(host).get(service):
                                cache.service_groups_by_host_and_service[host][service] = []

                            servicegroups = cache.service_groups_by_host_and_service[host][service]
                            servicegroups.append(servicegroup_name)

        return cache

    def _parse_members(self, line):
        matches = MEMBERS_REGEX.match(line)
        if matches:
            return matches.group(1).split(',')

        return []

    def _parse_hostgroup_name(self, line):
        matches = HOSTGROUP_NAME_REGEX.match(line)
        if matches:
            return matches.group(1)

        return []

    def _parse_servicegroup_name(self, line):
        matches = SERVICEGROUP_NAME_REGEX.match(line)
        if matches:
            return matches.group(1)

        return []

