import json

import urllib
import urllib2

class ZenossAPI():
    ROUTERS = {
        'MessagingRouter': 'messaging',
        'EventsRouter': 'evconsole',
        'ProcessRouter': 'process',
        'ServiceRouter': 'service',
        'DeviceRouter': 'device',
        'NetworkRouter': 'network',
        'TemplateRouter': 'template',
        'DetailNavRouter': 'detailnav',
        'ReportRouter': 'report',
        'MibRouter': 'mib',
        'ZenPackRouter': 'zenpack'
    }

    def __init__(self):
        self.url_opener = urllib2.build_opener(urllib2.HTTPCookieProcessor())
        self.reqCount = 1

    def update_config(self, config):
        self.zenoss_instance = config.get('host', 'localhost')

        # Contruct POST params and submit login.
        loginParams = urllib.urlencode(dict(
                       __ac_name = config['user'],
                       __ac_password = config['password'],
                       submitted = 'true',
                       came_from = self.zenoss_instance + '/zport/dmd'))
        self.url_opener.open(self.zenoss_instance + '/zport/acl_users/cookieAuthHelper/login', loginParams)

    def get_events_by_start_limit(self, open_only, start, limit):
        data = dict(start=start, limit=limit, dir='DESC', sort='stateChange')
        event_state = [0,1,2,3,4]
        if open_only:
            event_state = [0,1]
        data['params'] = dict(severity=[5,4,3,2], eventState=event_state)

        return self._router_request('EventsRouter', 'query', [data])['result']

    def _router_request(self, router, method, data=list()):
        if router not in self.ROUTERS:
            raise Exception('Router "' + router + '" not available.')

        # Contruct a standard URL request for API calls
        req = urllib2.Request(self.zenoss_instance + '/zport/dmd/' +
                              self.ROUTERS[router] + '_router')

        # NOTE: Content-type MUST be set to 'application/json' for these requests
        req.add_header('Content-type', 'application/json; charset=utf-8')

        # Convert the request parameters into JSON
        reqData = json.dumps([dict(
                    action=router,
                    method=method,
                    data=data,
                    type='rpc',
                    tid=self.reqCount)])

        # Increment the request count ('tid'). More important if sending multiple
        # calls in a single request
        self.reqCount += 1

        # Submit the request and convert the returned JSON to objects
        potential_json = self.url_opener.open(req, reqData).read()
        if self.is_json(potential_json):
            return json.loads(potential_json)
        else:
            raise Exception('Could not retrieve valid json from zenoss')

    def is_json(self, myjson):
        try:
            json_object = json.loads(myjson)
        except ValueError, e:
            return False
        return True