'''A module which implements a unix-like "tail" of a file.
A callback is made for every new line found in the file.  Options
specify whether the existing contents of the file should be read
or ignored.

@author Sean Reifschneider <jafo@tummy.com>
@author Shahar Kedar <shahar@gmail.com>
@version $Revision: 1.6 $

Released under the GPLv2 or any later version.

If a file is emptied or removed, the tail will continue reading lines
which are written in the new place.

Simple example:

    import tail

    def callback(line):
        print 'Line: "%s"' % string.rstrip(line)

    tail.tail('/var/log/all', callback).mainloop()
'''
#!/usr/bin/env python
import time
from file_utils import Tail


def mainloop(tail, sleepfor=1):
    """Loop forever processing activity on the tail object.
    This routine is intended to be called in programs which do not need
    to do other processing.  This routine never returns.

    @return Never returns
    @param sleepfor Seconds between processing (default is 5 seconds).
    """
    def callback(line, context):
        if line:
            print 'Line: "%s"' % line.rstrip()

    while True:
        tail.process(callback, None)
        time.sleep(sleepfor)


if __name__ == "__main__":

    tail = Tail('/tmp/nagios.log', -1)
    mainloop(tail)
